@@ -28,6 +28,7 @@ class Database(Traceback):
 		self.path = gfp.clean(path)
 		self.file_path = self.fp = FilePath(self.path)
 		if firestore == None:
+			self.database = dev0s.database.Database(self.path)
 			self.mode = "cache"
 			if self.live and not self.fp.exists():
 				dev0s.response.log(f"&ORANGE&Root permission&END& required to create database [{self.path}].")
@@ -44,13 +45,7 @@ class Database(Traceback):
 		if self.mode == "firestore":
 			return self.firestore.load(path)
 		elif self.mode == "cache":
-			path = gfp.clean(f"{self.path}/{path}")
-			try:
-				data = Files.load(path=path, format="json")
-			except Exception as e: return dev0s.response.error(str(e))
-			return dev0s.response.success(f"Successfully loaded [{path}].", {
-				"data":data,
-			})
+			return self.database.load(path=path)
 		else: raise exceptions.InvalidUsage(self.__traceback__(function="load")+f" Unknown mode [{self.mode}].")
 	def save(self, path=None, data=None, overwrite=False):
 		if path == None: return dev0s.response.error(self.__traceback__(function="save")+" Define parameter: [path].")
@@ -59,54 +54,17 @@ class Database(Traceback):
 		if self.mode == "firestore":
 			return self.firestore.save(path, data)
 		elif self.mode == "cache":
-			path = gfp.clean(f"{self.path}/{path}")
-			try:
-				if not Files.exists(path=gfp.base(path)): Files.create(path=gfp.base(path), directory=True)
-			except ValueError: a=1
-			if not overwrite:
-				def insert(old, new):
-					for key,value in new.items():
-						if isinstance(value, (dict, Dictionary)):
-							if key in old:
-								old[key] = insert(old[key], value)
-							else:
-								old[key] = value
-						elif isinstance(value, (list, Array)):
-							if key in old:
-								for i in value:
-									if i not in old[key]: old[key].append(i)
-							else:
-								old[key] = value
-						else:
-							old[key] = value
-					return old
-				try: old_data = Files.load(path=path, format="json")
-				except: old_data = {}
-				data = insert(old_data, data)
-			try:
-				Files.save(path=path, data=data, format="json")
-			except Exception as e: return dev0s.response.error(str(e))
-			return dev0s.response.success(f"Successfully saved [{path}].")
+			return self.database.save(path=path, data=data, overwrite=overwrite)
 		else: raise exceptions.InvalidUsage(self.__traceback__(function="save")+f" Unknown mode [{self.mode}].")
-	def delete(self, path=None, data=None):
+	def delete(self, path=None):
 		if path == None: return dev0s.response.error(self.__traceback__(function="delete")+" Define parameter: [path].")
 		path = gfp.clean(path)
 		if self.mode == "firestore":
 			return self.firestore.delete(path)
 		elif self.mode == "cache":
-			path = gfp.clean(f"{self.path}/{path}")
-			try:
-				Files.delete(path=path)
-			except Exception as e: return dev0s.response.error(str(e))
-			if Files.exists(path): return dev0s.response.error(f"Failed to delete [{path}].")
-			return dev0s.response.success(f"Successfully deleted [{path}].")
+			return self.database.delete(path=path)
 		else: raise exceptions.InvalidUsage(self.__traceback__(function="delete")+f" Unknown mode [{self.mode}].")
 
-	# join.
-	def join(self, path):
-		path = gfp.clean(path)
-		return gfp.clean(f"{self.path}/{path}")
-
 	# get names.
 	def names(self,
 		# the sub path (leave None to use the root path)
@@ -128,13 +86,22 @@ class Database(Traceback):
 
 		# cache.
 		elif self.mode == "cache":
-			for i in Directory(path=self.join(path)).paths(): names.append(gfp.name(i))
+			return self.database.names(path=path)
 
 		# handler.
 		return names
 
 		#
 
+	# path functions.
+	def subpath(self, fullpath):
+		return self.file_path.clean(path=fullpath.replace(self.path, ""), remove_double_slash=True)
+	def fullpath(self, subpath):
+		return self.file_path.clean(path=f"{self.path}/{subpath}", remove_double_slash=True)
+	def join(self, name=None, type=""):
+		return self.file_path.join(name, type)
+		#
+		
 	# representation.
 	def __str__(self):
 		return str(self.fp.path)
